/* Microbe3D: micro .obj 3D engine for Wavefront.OBJ files that use Microbe3D		*/
/* Microbe3D library: Alain THELLIER - Paris - FRANCE - July 2011 to 2013		*/
/* Microbe3D is small 3D engine that display directly 3D Wavefront .obj files		*/
/* It is maded as a library for programmers							*/
/* If you are a programmer and you want to have a nice 3D object(animated logo) 	*/
/* in your Amiga app then Microbe3D is done for you						*/

/* This file contains a demo for Microbe3D : a full 3D viewer with menus 		*/

#include <proto/Microbe3D.h>
#include <proto/graphics.h>		/* for Text() */
#include <proto/intuition.h>		/* for menus */
#include <intuition/intuition.h>	/* for cursor up,etc.. */

struct IntuitionBase*		IntuitionBase		=NULL;
struct GfxBase*			GfxBase			=NULL;
struct Library*			Microbe3DBase	=NULL;

#ifdef OS4
struct GraphicsIFace*		IGraphics		=NULL;
struct IntuitionIFace*		IIntuition		=NULL;
struct Microbe3DIFace*		IMicrobe3D		=NULL;
#endif
/*=================================================================*/
#define LIBOPEN(libbase,name,version)  libbase	=(void*)OpenLibrary(#name,(ULONG)version);						if(libbase==NULL) 	goto panic;
#define LIBOPEN4(interface,libbase)    interface=(void*)GetInterface((struct Library *)libbase, "main", 1, NULL);	if(interface==NULL)	goto panic;
#define LIBCLOSE(libbase)	if(libbase  !=NULL)	{  CloseLibrary((struct Library  *)libbase   );	libbase=NULL;	}
#define LIBCLOSE4(interface)	if(interface!=NULL)	{ DropInterface((struct Interface*)interface );	interface=NULL;	}
/*=================================================================*/
#define MAXMENU1 10
#define MAXMENU2 100
#define MAXMENU3 100
#define MAXMENU (MAXMENU1+MAXMENU2+MAXMENU3)
/*=================================================================*/
#define XLOOP(nbre) for(x=0;x<nbre;x++)
#define YLOOP(nbre) for(y=0;y<nbre;y++)
#define NLOOP(nbre) for(n=0;n<nbre;n++)
/*=================================================================*/
struct DemoMenu{
	UBYTE  name[40];
	UWORD level;		/* hierarchic level of the menu. 0=main menu */
	LONG  val;			/* send that value */
};
/*=================================================================*/
struct DemoData{
float WeldV,GridV,AngleN,SmoothAngle;
float RotX,RotY,RotZ;
BOOL EditX,EditY,EditZ,EditUV,Closed,Infos,Drag,Push,ShowStructure;
ULONG EditInum,EditOnum,EditMATnum,EditGRnum,EditLnum,EditVnum,EditColorNum;
UWORD EditThing;
float PixelResolution;
UWORD width,height;
UBYTE EditXYZ[4];
struct DemoMenu 	  M[MAXMENU];
struct Menu        M1[MAXMENU1];
struct MenuItem   MI2[MAXMENU2];
struct IntuiText  IT2[MAXMENU2];
struct MenuItem   MI3[MAXMENU3];
struct IntuiText  IT3[MAXMENU3];
#define MENULARGE 100
#define MENUHIGH  15
ULONG Mnb,M1nb,M2nb,M3nb;
} Demo;
/*=================================================================*/
float  ColorBlack[4]={0.0,0.0,0.0,1.0};
float  ColorWhite[4]={1.0,1.0,1.0,1.0};
float    ColorRed[4]={1.0,0.0,0.0,1.0};
float   ColorBlue[4]={0.0,0.0,1.0,1.0};
float ColorYellow[4]={1.0,1.0,0.0,1.0};
float  ColorGreen[4]={0.0,0.0,1.0,1.0};

float      ColorOrange[4]={1.0,1.0,0.1,1.0};
float   ColorLightBlue[4]={0.5,0.4,1.0,1.0};
float ColorLightYellow[4]={1.0,0.9,0.5,1.0};
float  ColorLightGreen[4]={0.5,0.4,1.0,1.0};
/*=================================================================*/
#ifdef DEBUGVERSION
#include "Microbe3D-debug.h" 
#endif
/*==========================================================================*/
void	 OSDrawText(APTR Scene,WORD x,WORD y,UBYTE *text)
{						/* draw a text in the window */
struct Window  *window;

	window=U3D_Get(Scene,U3D_AMIGAWINDOW,0);		/* hack Scene's window */
	if(window==NULL)
		return;

	SetAPen(window->RPort, 0) ;
	RectFill(window->RPort,x-3,y-9,x+8*strlen(text)+3,y+2);

	SetAPen(window->RPort, 2);
	Move(window->RPort,x-2,y-2);
	Text(window->RPort,(void*)text, strlen(text));

	SetAPen(window->RPort, 2);
	Move(window->RPort,x,y);
	Text(window->RPort,(void*)text, strlen(text));

	SetAPen(window->RPort, 1);
	Move(window->RPort,x-1,y-1);
	Text(window->RPort,(void*)text, strlen(text));

	SetAPen(window->RPort, 1);
}
/*=================================================================*/
void DemoDisplayInfos(APTR Scene)
{
APTR Object;
APTR Group;
APTR Instance;
APTR Material;
APTR GroupMaterial;
UBYTE mytext[512];
ULONG GRTRInb,GRnb,MATnb,TRInb,Vnb,UVnb,VNnb,Inb,Onb,n;
ULONG Fps,Fps50,Time,TrisDone,Tris50;
UBYTE *Sname;
UBYTE *Iname;
UBYTE *Oname;
UBYTE *GRname;
UBYTE *MATname;
UBYTE *GRMATname;

	if(!Demo.Infos) return;
	Demo.EditXYZ[0]=Demo.EditXYZ[1]=Demo.EditXYZ[2]='_';
	Demo.EditXYZ[3]=0;
	if(Demo.EditX) Demo.EditXYZ[0]='X';
	if(Demo.EditY) Demo.EditXYZ[1]='Y';
	if(Demo.EditZ) Demo.EditXYZ[2]='Z';

	Instance	=U3D_Get(Scene ,U3D_INSTANCE,Demo.EditInum);
	Object	=U3D_Get(Scene ,U3D_OBJECT  ,Demo.EditOnum);
	Material	=U3D_Get(Object,U3D_MATERIAL,Demo.EditMATnum);
	Group		=U3D_Get(Object,U3D_GROUP   ,Demo.EditGRnum);

	Sname	 =U3D_Name(Scene);
	Iname	 =U3D_Name(Instance);
	Oname	 =U3D_Name(Object);
	GRname =U3D_Name(Group);
	MATname=U3D_Name(Material);
	GRname =U3D_Name(Group);

	Onb		=U3D_QueryValue(Scene ,U3D_OBJECT);
	Inb		=U3D_QueryValue(Scene ,U3D_INSTANCE);

	MATnb		=U3D_QueryValue(Object,U3D_MATERIAL);
	TRInb		=U3D_QueryValue(Object,U3D_TRIANGLE);
	Vnb		=U3D_QueryValue(Object,U3D_VERTEX);
	UVnb		=U3D_QueryValue(Object,U3D_UV);
	VNnb		=U3D_QueryValue(Object,U3D_VNORMAL);
	GRnb		=U3D_QueryValue(Object,U3D_GROUP);

	GRTRInb 	=U3D_QueryValue(Group,U3D_TRIANGLE);

	Fps		=U3D_QueryValue(Scene,U3D_FPS);
	Fps50		=U3D_QueryValue(Scene,U3D_FPS50);
	Time		=U3D_QueryValue(Scene,U3D_TIME);
	TrisDone	=U3D_QueryValue(Scene,U3D_TRISDONE);
	Tris50	=U3D_QueryValue(Scene,U3D_TRIS50);

	GroupMaterial=U3D_Get(Group,U3D_MATERIAL,0);		/* Group material */
	GRMATname=U3D_Name(GroupMaterial);

	sprintf(mytext,"%s: %ld Fps(On 50 frames:%ld) Time %ld TrisDone %ld (%ld) ",Sname,Fps,Fps50,Time,TrisDone,Tris50);
	OSDrawText(Scene,3,14*1,mytext);

	sprintf(mytext,"Object<%s> got TRI:%ld V:%ld VN:%ld UV:%ld Group:%ld",Oname,TRInb,Vnb,VNnb,UVnb,GRnb);
	OSDrawText(Scene,3,14*2,mytext);

	sprintf(mytext,"Group%ld<%s> got %ld Tris use Material<%s>. SmoothAngle:%ld Light(%ld)Color(%ld)",Demo.EditGRnum,GRname,GRTRInb,GRMATname,(LONG)Demo.SmoothAngle,Demo.EditLnum,Demo.EditColorNum);
	OSDrawText(Scene,3,14*3,mytext);

	sprintf(mytext,"EDIT[%c]: [%s] Ins:%s(%ld/%ld) Obj:%s(%ld/%ld) Group:%s(%ld/%ld) Mat:%s(%ld/%ld)",Demo.EditThing,Demo.EditXYZ,Iname,Demo.EditInum+1,Inb,Oname,Demo.EditOnum+1,Onb,GRname,Demo.EditGRnum+1,GRnb,MATname,Demo.EditMATnum+1,MATnb);
	OSDrawText(Scene,3,14*4,mytext);

#ifdef DEBUGVERSION
	DebugProg(Scene);
#endif
}
/*=================================================================*/
APTR NiceRead(APTR Scene,UBYTE *filename)
{
/* load an object and adjust it to screen size*/
APTR Object;
ULONG VNnb;
BOOL CleanObject=TRUE;
UBYTE name[40];
APTR Instance;
APTR Material;
APTR Group;
APTR EditMaterial;
ULONG Inb;
ULONG Onb;

	Object=U3D_Read(Scene,filename);

	if(!Object)		
		return(Object);

U3D_Write(Object,"RAM:demo-view-debug.obj");

	if(CleanObject)
	{
	U3D_Unitize(Object,2.0);							/* fit to screen (ie -1.0 +1.0) */

	VNnb=U3D_QueryValue(Object,U3D_VNORMAL);
	if(VNnb==0)										/*  do vertex-normals if absents */
		U3D_Normals(Object,80.0);

	if(Demo.width > Demo.height)
		Demo.PixelResolution = 1.0/(float)Demo.width;
	else
		Demo.PixelResolution = 1.0/(float)Demo.height;
	U3D_GridWeld(Object,Demo.PixelResolution,0.001,0.001);		/* will remove the double points not visible on screen & do smooth normals */
	}

	Inb=U3D_QueryValue(Scene ,U3D_INSTANCE);
	sprintf(name,"Instance%ld",Inb);
	Instance=U3D_AddInstance(Scene,Object,name);
	Inb=U3D_QueryValue(Scene ,U3D_INSTANCE);
	Demo.EditInum=Inb-1;

	Onb=U3D_QueryValue(Scene ,U3D_OBJECT);
	Demo.EditOnum=Onb-1;

	Demo.EditGRnum=0;
	Group=U3D_Get(Object,U3D_GROUP,Demo.EditGRnum);

	Demo.EditMATnum=0;
	Material=U3D_Get(Group,U3D_MATERIAL,Demo.EditMATnum);
	while(EditMaterial!=Material)				/* Set Demo.EditMATnum to this Material */ 
		{ EditMaterial=U3D_Get(Object,U3D_MATERIAL,Demo.EditMATnum++); }
	Demo.EditMATnum--;

	return(Object);
}
/*=================================================================*/
BOOL DemoProg(APTR Scene)
{
APTR Object;
APTR Instance;
APTR Group;
APTR Material;
APTR Material2;
APTR EditMaterial;
APTR Light;
APTR EditThing;
APTR InstanceThing;
APTR Vertex;
Vertex3D *V;
Vertex3D* MouseVector;
Vertex3D dimension,min,max;
ULONG key,MouseButtons,VanillaKey,MenuCommand,GadgetID;
LONG n,Inb,Onb,MATnb;
float step;
UBYTE name[40];
BOOL ok;


	VanillaKey	=U3D_QueryValue(Scene,U3D_VANILLAKEY);
	MenuCommand	=U3D_QueryValue(Scene,U3D_MENUCOMMAND);
	GadgetID	=U3D_QueryValue(Scene,U3D_GADGETID);


	key=VanillaKey;
	if(MenuCommand)
		key=MenuCommand;

	Object	=U3D_Get(Scene,U3D_OBJECT,Demo.EditOnum);

	ok=(Object!=NULL);
	if(key==27)
		ok=TRUE;	
	if(key=='r')	
		ok=TRUE;	
	if(key=='f')	
		ok=TRUE;
	if(!ok)
		return;

	Instance	=U3D_Get(Scene,U3D_INSTANCE,Demo.EditInum);
	Material	=U3D_Get(Object,U3D_MATERIAL,Demo.EditMATnum);
	Group		=U3D_Get(Object,U3D_GROUP,Demo.EditGRnum);
	Light		=U3D_Get(Scene,U3D_LIGHT,Demo.EditLnum);
	Vertex	=U3D_Get(Object,U3D_VERTEX,Demo.EditVnum);


	if(Material==NULL)
		Demo.EditMATnum=0;
	Material=U3D_Get(Object,U3D_MATERIAL,Demo.EditMATnum);

	if(Group==NULL)
		Demo.EditGRnum=0;
	Group=U3D_Get(Object,U3D_GROUP,Demo.EditGRnum);

	if(Vertex==NULL)
		Demo.EditVnum=0;
	Vertex=U3D_Get(Object,U3D_VERTEX,Demo.EditVnum);

	if(Object==NULL)
		Demo.EditOnum=0;
	Object=U3D_Get(Scene,U3D_OBJECT,Demo.EditOnum);

	if(Instance==NULL)
		Demo.EditInum=0;
	Instance=U3D_Get(Scene,U3D_INSTANCE,Demo.EditInum);


	EditThing=NULL;
	if(Demo.EditThing=='S')
		EditThing=Scene;
	if(Demo.EditThing=='I')
		EditThing=Instance;
	if(Demo.EditThing=='O')
		EditThing=Object;
	if(Demo.EditThing=='L')
		EditThing=Light;
	if(Demo.EditThing=='p')
		EditThing=Light;
	if(Demo.EditThing=='V')
		EditThing=Vertex;

	Demo.Closed=FALSE;

	switch(key)
	{
	case 27:
		Demo.Closed=TRUE;		break;

	case 'f':
		Demo.Infos=!Demo.Infos;		break;

	case 'M':
		Demo.ShowStructure=!Demo.ShowStructure;
		if( Demo.ShowStructure) U3D_QueryValue(Scene,U3D_SHOWSTRUCTURE);
		if(!Demo.ShowStructure) U3D_QueryValue(Scene,U3D_HIDESTRUCTURE);
		break;

	case 'm':
		Demo.EditMATnum++;
		Material=U3D_Get(Object,U3D_MATERIAL,Demo.EditMATnum);
		if(Material==NULL)
			Demo.EditMATnum=0;
		Material=U3D_Get(Object,U3D_MATERIAL,Demo.EditMATnum);
		break;

	case 'g':
		Demo.EditGRnum++;
		Group=U3D_Get(Object,U3D_GROUP,Demo.EditGRnum);
		if(Group==NULL)
			Demo.EditGRnum=0;
		Group=U3D_Get(Object,U3D_GROUP,Demo.EditGRnum);
		break;

	case ':':
		Demo.EditVnum++;
		Vertex=U3D_Get(Object,U3D_VERTEX,Demo.EditVnum);
		if(Vertex==NULL)
			Demo.EditVnum=0;
		Vertex=U3D_Get(Object,U3D_VERTEX,Demo.EditVnum);
		break;

	case 'o':
		Demo.EditOnum++;
		Object=U3D_Get(Scene,U3D_OBJECT,Demo.EditOnum);
		if(Object==NULL)
			Demo.EditOnum=0;
		Object=U3D_Get(Scene,U3D_OBJECT,Demo.EditOnum);
		break;

	case 'i':
		Demo.EditInum++;
		Instance=U3D_Get(Scene,U3D_INSTANCE,Demo.EditInum);
		if(Instance==NULL)
			Demo.EditInum=0;
		Instance=U3D_Get(Scene,U3D_INSTANCE,Demo.EditInum);
		Demo.EditThing='I';
		break;

	case 'l':
		Demo.EditLnum++;
		if(Demo.EditLnum>8)
			Demo.EditLnum=0;
		break;

	case '!':
		Inb=U3D_QueryValue(Scene,U3D_INSTANCE);
		NLOOP(Inb)
		{
		Instance	 =U3D_Get(Scene ,U3D_INSTANCE,(Inb-1)-n);
		InstanceThing=U3D_Get(Instance,U3D_THING,0);
		if(InstanceThing==Object)
				U3D_Delete(Instance);		/* remove instances using this object */
		}
		U3D_Delete(Object);
/* get last Instance */
		Inb=U3D_QueryValue(Scene ,U3D_INSTANCE);
		Demo.EditInum=Inb-1;
		Instance	=U3D_Get(Scene ,U3D_INSTANCE,Demo.EditInum);
/* get last Object */
		Onb=U3D_QueryValue(Scene ,U3D_OBJECT);
		Demo.EditOnum=Onb-1;
		Object	=U3D_Get(Scene ,U3D_OBJECT  ,Demo.EditOnum);
/* get first Group in this Object */
		Demo.EditGRnum=0;
		Group=U3D_Get(Object,U3D_GROUP,Demo.EditGRnum);
/* get Material = Object->Group->Material */
		Demo.EditMATnum=0;
		Material=U3D_Get(Group,U3D_MATERIAL,Demo.EditMATnum);
		while(EditMaterial!=Material)				/* Set Demo.EditMATnum to this Material */ 
			{ EditMaterial=U3D_Get(Object,U3D_MATERIAL,Demo.EditMATnum++); }
		Demo.EditMATnum--;
		break;

	case 'r':
		Object=NiceRead(Scene,NULL);
		break;

	case 'R':
		U3D_ReverseWinding(Object);
		break;

	case '/':
		U3D_Scale(EditThing,0.8,0.8,0.8,U3D_CHANGE);
		break;

	case '*':
		U3D_Scale(EditThing,1.25,1.25,1.25,U3D_CHANGE);
		break;

	case 'v':
		Demo.WeldV -= 0.01;
		U3D_Weld(Object, Demo.WeldV,0.0,0.0);
		break;

	case 'V':
		Demo.WeldV += 0.01;
		U3D_Weld(Object, Demo.WeldV,0.0,0.0);
		break;

	case '=':
		U3D_Weld(Object,0.0,0.0,0.0);
		break;

	case 'k':
		Demo.GridV=Demo.GridV+Demo.PixelResolution;
		U3D_GridWeld(Object,Demo.GridV,Demo.AngleN,0.0);
		break;
	case 'K':
		Demo.AngleN=Demo.AngleN+0.5;
		U3D_GridWeld(Object,Demo.GridV,Demo.AngleN,0.0);
		break;

	case '"':
		Demo.GridV=Demo.PixelResolution;
		U3D_Scale(Object,0.8,0.8,0.8,U3D_CHANGE);
		U3D_GridWeld(Object,Demo.GridV,Demo.AngleN,0.0);
		break;


	case '-':
		Demo.SmoothAngle -= 1.0;
		U3D_Normals(Object,Demo.SmoothAngle);
		break;

	case '+':
		Demo.SmoothAngle += 1.0;
		U3D_Normals(Object,Demo.SmoothAngle);
		break;


	case 'U':
		U3D_Unitize(Object,2.0);
		U3D_Normals(Object,Demo.SmoothAngle);
		U3D_GridWeld(Object,Demo.PixelResolution,0.0,0.0);		/* will remove the double points not visible on screen & do smooth normals */
		break;

	case 'u':
		U3D_Unitize(Object,2.0);
		break;

	case 'F':
		Demo.SmoothAngle = 0.0;
		U3D_Normals(Object,Demo.SmoothAngle);
		break;

	case 'L':
		if(Demo.EditX==1.0) U3D_SetSpecialMaterial(Object,Material,NULL,U3D_TEXLINEARX);	/* x linear texturing */
		if(Demo.EditY==1.0) U3D_SetSpecialMaterial(Object,Material,NULL,U3D_TEXLINEARY);	/* y linear texturing */
		if(Demo.EditZ==1.0) U3D_SetSpecialMaterial(Object,Material,NULL,U3D_TEXLINEARZ);	/* z linear texturing */
		break;

	case 'O':
		U3D_SetSpecialMaterial(Object,Material,NULL,U3D_TEXSPHERIC);    /* sphere texturing */
		break;

	case 'c':
		Material2=U3D_Find(Object,U3D_MATERIAL,"mat_cubemapxpos");
		U3D_SetSpecialMaterial(Object,Material,Material2,U3D_TEXCUBIC );    /* cubic texturing (static)*/
		break;

	case 'C':
		Material2=U3D_Find(Object,U3D_MATERIAL,"mat_cubemapxpos");
		U3D_SetSpecialMaterial(Object,Material,Material2,U3D_MATENVCUBE);	/* cub env (dynamic) */
		break;

	case 'T':
		U3D_SetSpecialMaterial(Object,Material,NULL,U3D_MATTOON);	/* toon shader (dynamic) */
		break;

	case 'Q':
		U3D_SetSpecialMaterial(Object,Material,NULL,U3D_MATZGREY);	/*  z to grey (dynamic) */
		break;

	case 'e':
		U3D_SetLight(Light,TRUE,ColorRed);
		Material2=U3D_Find(Object,U3D_MATERIAL,"mat_bump");
		U3D_SetSpecialMaterial(Object,Material,Material2,U3D_MATBUMP);	/* bump mapping */
		break;

	case '|':
		U3D_SetSpecialMaterial(Object,Material,NULL,U3D_MATWIREFRAME);
		break;
	case '>':
		U3D_SetSpecialMaterial(Object,Material,NULL,U3D_MATPLOT);
		break;
	case '#':
		U3D_SetSpecialMaterial(Object,Material,NULL,U3D_MATSOLID);
		break;

	case 'p':
		U3D_SetSpecialMaterial(Object,Material,NULL,U3D_MATFLAT);
		break;
	case 'n':
		U3D_SetSpecialMaterial(Object,Material,NULL,U3D_MATSTANDARD);
		break;
	case '':
		U3D_SetSpecialMaterial(Object,Material,NULL,U3D_MATTEXTUREDFLAT);
		break;
	case '~':
		U3D_SetSpecialMaterial(Object,Material,NULL,U3D_MATTEXTURED);
		break;

	case 'G':
		U3D_SetSpecialMaterial(Object,Material,Group,U3D_MATTOGROUP);
		break;

	case '<':
		MATnb		=U3D_QueryValue(Object,U3D_MATERIAL);
		NLOOP(MATnb)
			{
			Material=U3D_Get(Object,U3D_MATERIAL,n);
			U3D_SetSpecialMaterial(Object,Material,NULL,U3D_MATLIGHTFAST);
			}
		Material=U3D_Get(Object,U3D_MATERIAL,Demo.EditMATnum);
		break;

	case 'b':
		U3D_Box(Object,1.0,1.0,1.0);
		break;

	case 'B':
		U3D_Dimensions(Object,&min,&max,&dimension);
		printf("Min    %ld %ld %ld\n",(LONG)(100.0*min.x),(LONG)(100.0*min.y),(LONG)(100.0)*min.z);
		printf("Max    %ld %ld %ld\n",(LONG)(100.0*max.x),(LONG)(100.0*max.y),(LONG)(100.0)*max.z);
		printf("Dim    %ld %ld %ld\n",(LONG)(100.0*dimension.x),(LONG)(100.0*dimension.y),(LONG)(100.0)*dimension.z);
		break;

	case '(':
		step=+0.05;
		U3D_Translate(EditThing,step*Demo.EditX,step*Demo.EditY,step*Demo.EditZ,U3D_CHANGE);
		if(Demo.EditThing=='V')
			{
			V=Vertex;
			V->x=V->x + step*Demo.EditX ;
			V->y=V->y + step*Demo.EditY ;
			V->z=V->z + step*Demo.EditZ ;
			}
		break;

	case ')':
		step=-0.05;
		U3D_Translate(EditThing,step*Demo.EditX,step*Demo.EditY,step*Demo.EditZ,U3D_CHANGE);
		if(Demo.EditThing=='V')
			{
			V=Vertex;
			V->x=V->x + step*Demo.EditX ;
			V->y=V->y + step*Demo.EditY ;
			V->z=V->z + step*Demo.EditZ ;
			}
		break;

	case '{':
		step=+1.0;
		U3D_Rotate(EditThing,step*Demo.EditX,step*Demo.EditY,step*Demo.EditZ,U3D_CHANGE);
		break;

	case '}':
		step=-1.0;
		U3D_Rotate(EditThing,step*Demo.EditX,step*Demo.EditY,step*Demo.EditZ,U3D_CHANGE);
		break;

		break;

	case 'I':
		Inb=U3D_QueryValue(Scene ,U3D_INSTANCE);
		sprintf(name,"Instance%ld",Inb);
		Instance=U3D_AddInstance(Scene,Object,name);
		Inb=U3D_QueryValue(Scene ,U3D_INSTANCE);
		Demo.EditInum=Inb-1;
		break;

	case 'N':
		Demo.EditColorNum++;
		if(Demo.EditColorNum>9)
			Demo.EditColorNum=0;

		printf("Light%ld (%ld) set to Color%ld : ",Demo.EditLnum,Light,Demo.EditColorNum);
		if(Demo.EditColorNum==0) {U3D_SetLight(Light,FALSE,ColorBlack); printf(" Black \n");}
		if(Demo.EditColorNum==1) {U3D_SetLight(Light,TRUE ,ColorWhite);printf(" White \n");}
		if(Demo.EditColorNum==2) {U3D_SetLight(Light,TRUE ,ColorRed);printf(" Red \n");}
		if(Demo.EditColorNum==3) {U3D_SetLight(Light,TRUE ,ColorBlue);printf(" Blue \n");}
		if(Demo.EditColorNum==4) {U3D_SetLight(Light,TRUE ,ColorYellow);printf(" Yellow \n");}
		if(Demo.EditColorNum==5) {U3D_SetLight(Light,TRUE ,ColorGreen);printf(" Green \n");}

		if(Demo.EditColorNum==6) {U3D_SetLight(Light,TRUE ,ColorOrange);printf(" Orange \n");}
		if(Demo.EditColorNum==7) {U3D_SetLight(Light,TRUE ,ColorLightYellow);printf(" LightYellow \n");}
		if(Demo.EditColorNum==8) {U3D_SetLight(Light,TRUE ,ColorLightGreen);printf(" LightGreen \n");}
		if(Demo.EditColorNum==9) {U3D_SetLight(Light,TRUE ,ColorLightBlue);printf(" LightBlue \n");}
		break;

	case 'w':
		U3D_Write(Object,NULL);
		break;

	case 'x':	if(Demo.EditX ==0.0) Demo.EditX =1.0;	else Demo.EditX =0.0; break;
	case 'y':	if(Demo.EditY ==0.0) Demo.EditY =1.0;	else Demo.EditY =0.0; break;
	case 'z':	if(Demo.EditZ ==0.0) Demo.EditZ =1.0;	else Demo.EditZ =0.0; break;
	case 't':	if(Demo.EditUV==0.0) Demo.EditUV=1.0;	else Demo.EditUV=0.0; break;
	case ' ':
		if(Demo.EditThing=='I') {Demo.EditThing='O'; printf("Editing: Object\n"); break;}
		if(Demo.EditThing=='O') {Demo.EditThing='S'; printf("Editing: Scene\n"); break;}
		if(Demo.EditThing=='S') {Demo.EditThing='p'; printf("Editing: Light pos\n"); break;}
		if(Demo.EditThing=='p') {Demo.EditThing='L'; printf("Editing: Light dir\n"); break;}
		if(Demo.EditThing=='L') {Demo.EditThing='V'; printf("Editing: Vertex\n"); break;}
		if(Demo.EditThing=='V') {Demo.EditThing='I'; printf("Editing: Instance\n"); break;}
		break;

	case 'A':
		if(Demo.EditX ==1.0)  U3D_SwapAxis(Object,U3D_SWAPYZ);
		if(Demo.EditY ==1.0)  U3D_SwapAxis(Object,U3D_SWAPXZ);
		if(Demo.EditZ ==1.0)  U3D_SwapAxis(Object,U3D_SWAPXY);
		if(Demo.EditUV==1.0)  U3D_SwapAxis(Object,U3D_SWAPUV);
		break;


	case 'q':
		U3D_Rotate(Scene,0.0,90.0,0.0,U3D_RESET);
		break;

	case 'h':
		U3D_QueryValue(Scene,9999);
		break;

	case '0':
		step=+5.0;
		U3D_Rotate(EditThing,0.0,0.0,step,U3D_CHANGE);
		break;
	case '.':
		step=+5.0;
		U3D_Rotate(EditThing,0.0,0.0,-step,U3D_CHANGE);
		break;

	default:
		break;
	}

/*-----------------------------------------------------------*/
	MouseVector=U3D_Get(Scene,U3D_MOUSEVECTOR,0);
	Demo.RotX=Demo.EditX*MouseVector->y*180.0;	/* swap x y so rotation follow mouse direction */
	Demo.RotY=Demo.EditY*MouseVector->x*180.0;
	Demo.RotZ=Demo.EditZ*MouseVector->z*180.0;	/* in fact MouseVector.z=MouseVector.x */

	MouseButtons=U3D_QueryValue(Scene,U3D_MOUSEBUTTONS);
	if (MouseButtons== SELECTDOWN)
		Demo.Drag=1;
	if (MouseButtons== SELECTUP)
		Demo.Drag=0;

	if(Demo.Drag)
	{
	if(Demo.EditThing=='p')
		U3D_Translate(EditThing,Demo.RotY/180.0,Demo.RotX/180.0,0.1,U3D_RESET);

	if(Demo.EditThing!='p')
		U3D_Rotate(EditThing,Demo.RotX,Demo.RotY,Demo.RotZ,U3D_RESET);
	}
/*-----------------------------------------------------------*/
	key =U3D_QueryValue(Scene,U3D_RAWKEY);
	step=+5.0;
	switch(key)
	{
	case CURSORUP:
		U3D_Rotate(EditThing,step,0.0,0.0,U3D_CHANGE);
		break;
	case CURSORDOWN:
		U3D_Rotate(EditThing,-step,0.0,0.0,U3D_CHANGE);
		break;
	case CURSORRIGHT:
		U3D_Rotate(EditThing,0.0,step,0.0,U3D_CHANGE);
		break;
	case CURSORLEFT:
		U3D_Rotate(EditThing,0.0,-step,0.0,U3D_CHANGE);
		break;
	default:
		break;
	}
/*-----------------------------------------------------------*/
	return(!Demo.Closed);
}
/*=================================================================*/
void AddMenu1(struct DemoMenu *M)
{
ULONG n;

	n=Demo.M1nb;
	if(n>MAXMENU1)
		return;

	Demo.M1[n].NextMenu =&Demo.M1[n+1];
	Demo.M1[n].FirstItem=&Demo.MI2[Demo.M2nb];

	if(Demo.M2nb>=1)
		Demo.MI2[Demo.M2nb-1].NextItem=NULL;

	Demo.M1[n].LeftEdge=0;
	Demo.M1[n].TopEdge=0;

	if(n>0)
	{
	Demo.M1[n].LeftEdge=Demo.M1[n-1].LeftEdge+MENULARGE;
	}

	Demo.M1[n].Width=MENULARGE;
	Demo.M1[n].Height=MENUHIGH;
	Demo.M1[n].Flags=MENUENABLED;
	Demo.M1[n].MenuName=M->name;

	Demo.M1[n].JazzX=0;
	Demo.M1[n].JazzY=0;
	Demo.M1[n].BeatX=0;
	Demo.M1[n].BeatY=0;

	Demo.M1nb++;
}
/*=================================================================*/
void AddMenu2(struct DemoMenu *M)
{
ULONG n;

	n=Demo.M2nb;
	if(n>MAXMENU2)
		return;

	Demo.MI2[n].NextItem=&Demo.MI2[n+1];
	Demo.MI2[n].SubItem =NULL;

/*	Demo.MI2[n].SubItem =&Demo.MI3[Demo.M3nb]; */
/*	if(Demo.M3nb>=1)					 */
/*		Demo.MI3[Demo.M3nb-1].NextItem=NULL; */


	Demo.MI2[n].LeftEdge=0;
	Demo.MI2[n].TopEdge=0;

	if(n>0)
	{
	if(Demo.MI2[n-1].NextItem!=NULL)
		{
		Demo.MI2[n].TopEdge=Demo.MI2[n-1].TopEdge+MENUHIGH;
		Demo.MI2[n].LeftEdge=Demo.MI2[n-1].LeftEdge;
		}
	else	
		{
		Demo.MI2[n].TopEdge=0;
		Demo.MI2[n].LeftEdge=Demo.MI2[n-1].LeftEdge;
		}
	}

	Demo.MI2[n].Width=MENULARGE;
	Demo.MI2[n].Height=MENUHIGH;
	Demo.MI2[n].Flags=ITEMTEXT|ITEMENABLED|HIGHCOMP;
	Demo.MI2[n].MutualExclude=0;
	Demo.MI2[n].ItemFill=&Demo.IT2[n];
	Demo.MI2[n].SelectFill=&Demo.IT2[n];
	Demo.MI2[n].Command=M->val;
	Demo.MI2[n].NextSelect=0;

	Demo.IT2[n].FrontPen=0;
	Demo.IT2[n].BackPen=1;
	Demo.IT2[n].DrawMode=JAM2;
	Demo.IT2[n].LeftEdge=4;
	Demo.IT2[n].TopEdge=2;
	Demo.IT2[n].ITextFont=NULL;
	Demo.IT2[n].IText=M->name;
	Demo.IT2[n].NextText=NULL;

	Demo.M2nb++;
}
/*=================================================================*/
void AddMenu3(struct DemoMenu *M)
{
ULONG n;

	n=Demo.M3nb;
	if(n>MAXMENU3)
		return;

	Demo.MI3[n].NextItem=&Demo.MI3[n+1];
	Demo.MI3[n].SubItem=NULL;

	Demo.MI3[n].LeftEdge=0;
	if(n>=1)
		Demo.MI3[n].TopEdge=Demo.MI3[n-1].TopEdge+MENUHIGH;
	else
		Demo.MI3[n].TopEdge=0;

	Demo.MI3[n].Width=MENULARGE;
	Demo.MI3[n].Height=MENUHIGH;
	Demo.MI3[n].Flags=ITEMTEXT|ITEMENABLED|HIGHCOMP;
	Demo.MI3[n].MutualExclude=0;
	Demo.MI3[n].ItemFill=&Demo.IT2[n];
	Demo.MI3[n].SelectFill=&Demo.IT2[n];
	Demo.MI3[n].Command=M->val;
	Demo.MI3[n].NextSelect=0;

	Demo.IT3[n].FrontPen=0;
	Demo.IT3[n].BackPen=1;
	Demo.IT3[n].DrawMode=JAM2;
	Demo.IT3[n].LeftEdge=4;
	Demo.IT3[n].TopEdge=2;
	Demo.IT3[n].ITextFont=NULL;
	Demo.IT3[n].IText=M->name;
	Demo.IT3[n].NextText=NULL;

	Demo.M3nb++;
}
/*=================================================================*/
LONG AddMenu(UWORD level,UBYTE *name,LONG val)
{
ULONG n;
UBYTE keyname[10];

	n=Demo.Mnb;
	if(n>MAXMENU)
		return;

	strcpy(Demo.M[n].name,name);
	if(isprint(val))
	{
	sprintf(keyname," [%c]",val);
	strcat(Demo.M[n].name,keyname);
	}	

	Demo.M[n].level=level;
	Demo.M[n].val  =val;
	Demo.Mnb++;
}
/*==========================================================================*/
void DemoSetMenus(APTR Scene)
{
ULONG n;
struct Window  *window;

	Demo.Mnb =0;
	Demo.M1nb=0;
	Demo.M2nb=0;
	Demo.M3nb=0;

	AddMenu(1,"Scene",0);
		AddMenu(2,"Read:   Object"		,'r');
		AddMenu(2,"Write:  Object"		,'w');
		AddMenu(2,"Delete: Object"		,'!');
		AddMenu(2,"Show Infos"			,'f');
		AddMenu(2,"Show Structures"		,'M');
		AddMenu(2,"Quit"				, 27);

	AddMenu(1,"Edit",0);
		AddMenu(2,"Next Instance"		,'i');
		AddMenu(2,"Next Object"			,'o');
		AddMenu(2,"Next Group"			,'g');
		AddMenu(2,"Next Material"		,'m');
		AddMenu(2,"Next Light"			,'l');
		AddMenu(2,"_________________________",0);
		AddMenu(2,"Edit I/O/S/Light/LightPos",' ');
		AddMenu(2,"_________________________",0);
		AddMenu(2,"Edit X"			,'x');
		AddMenu(2,"Edit Y"			,'y');
		AddMenu(2,"Edit Z"			,'z');
		AddMenu(2,"Edit UV"			,'t');
		AddMenu(2,"_________________________",0);
		AddMenu(2,"Translate: +0.05"		,'(');
		AddMenu(2,"Translate: -0.05"		,')');
		AddMenu(2,"_________________________",0);
		AddMenu(2,"Scale:  80%"			,'/');
		AddMenu(2,"Scale: 125%"			,'*');
		AddMenu(2,"_________________________",0);
		AddMenu(2,"Rotate:  +1"		,'{');
		AddMenu(2,"Rotate:  -1"		,'}');

	AddMenu(1,"Object",0);
		AddMenu(2,"Normals: angle +1"	,'+');
		AddMenu(2,"Normals: angle -1"	,'-');
		AddMenu(2,"Normals: angle 0 : face",'F');
		AddMenu(2,"_________________________",0);
		AddMenu(2,"Weld: WeldV +0.01"		,'V');
		AddMenu(2,"Weld: WeldV -0.01"		,'v');
		AddMenu(2,"Weld: 0.0:duplicate only",'=');
		AddMenu(2,"_________________________",0);
		AddMenu(2,"GridWeld: V  more"		,'k');
		AddMenu(2,"GridWeld: VN more"		,'K');
		AddMenu(2,"GridWeld: pix. res."	,'"');
		AddMenu(2,"_________________________",0);
		AddMenu(2,"Clean to screen"		,'U');
		AddMenu(2,"Unitize:"			,'u');
		AddMenu(2,"Box: 1 1 1"			,'b');
		AddMenu(2,"Dimensions:"			,'B');
		AddMenu(2,"ReverseWinding:"		,'R');
		AddMenu(2,"_________________________",0);
		AddMenu(2,"Next Vertex"			,':');
		AddMenu(2,"_________________________",0);
		AddMenu(2,"Next Object"			,'o');

	AddMenu(1,"Instance",0);
		AddMenu(2,"AddInstance:"		,'I');
		AddMenu(2,"_________________________",0);
		AddMenu(2,"Next Instance"		,'i');

	AddMenu(1,"Material",0);
		AddMenu(2,"TEX LINEAR X/Y/Z"		,'L');
		AddMenu(2,"TEX SPHERIC"			,'O');
		AddMenu(2,"TEX CUBIC"			,'c');
		AddMenu(2,"_________________________",0);
		AddMenu(2,"MAT ENV CUBE"		,'C');
		AddMenu(2,"MAT TOON"			,'T');
		AddMenu(2,"MAT BUMP"			,'e');
		AddMenu(2,"MAT Z GREY"			,'Q');
		AddMenu(2,"_________________________",0);
		AddMenu(2,"MAT WIREFRAME"		,'|');
		AddMenu(2,"MAT PLOT"			,'>');
		AddMenu(2,"MAT SOLID"			,'#');
		AddMenu(2,"_________________________",0);
		AddMenu(2,"MAT FLAT"			,'p');
		AddMenu(2,"MAT STANDARD"		,'n');
		AddMenu(2,"MAT TEXTUREDFLAT"		,'');
		AddMenu(2,"MAT TEXTURED"		,'~');
		AddMenu(2,"_________________________",0);
		AddMenu(2,"MAT TO GROUP"		,'G');
		AddMenu(2,"_________________________",0);
		AddMenu(2,"Next Material"		,'m');

	AddMenu(1,"Light",0);
		AddMenu(2,"SetLight:no/white/red/...",'N');
		AddMenu(2,"MAT LIGHT FAST(All Mats) ",'<');
		AddMenu(2,"_________________________",0);
		AddMenu(2,"Next Light"			,'l');

#ifdef DEBUGVERSION
	AddMenu(1,"Hack W3D states",0);
		AddMenu(2,"ZUpdate"		,'Z');
		AddMenu(2,"Z_NEVER"		,'1');
		AddMenu(2,"Z_LESS"		,'2');
		AddMenu(2,"Z_GEQUAL"		,'3');
		AddMenu(2,"Z_LEQUAL"		,'4');
		AddMenu(2,"Z_GREATER"		,'5');
		AddMenu(2,"Z_NOTEQUAL"		,'6');
		AddMenu(2,"Z_EQUAL"		,'7');
		AddMenu(2,"Z_ALWAYS"		,'8');
		AddMenu(2,"_________________________",0);
		AddMenu(2,"Point/LineSize++"	,'P');
		AddMenu(2,"CullingMode"		,'H');
		AddMenu(2,"BlendMode--"		,'S');
		AddMenu(2,"BlendMode++"		,'D');
		AddMenu(2,"srcfunc"		,'s');
		AddMenu(2,"dstfunc"		,'d');
		AddMenu(2,"TexEnvMode"		,'E');

	AddMenu(1,"Hack & Debug",0);
		AddMenu(2,"debug"			,'a');
		AddMenu(2,"w3dpatch"		,'W');
		AddMenu(2,"DrawScene"		,'j');
		AddMenu(2,"DrawStuff"		,'J');
		AddMenu(2,"ClearZbuffer"	,'');
		AddMenu(2,"ClearBitmap"		,'@');
		AddMenu(2,"BackGround"		,'');
		AddMenu(2,"_________________________",0);
		AddMenu(2,"Next Triangle"		,'9');
		AddMenu(2,"Reverse Triangle"		,'%');
		AddMenu(2,"Remove  Triangle"		,'#');
#endif

	NLOOP(Demo.Mnb)
		{
		if(Demo.M[n].level==1)
			AddMenu1(&Demo.M[n]);
		if(Demo.M[n].level==2)
			AddMenu2(&Demo.M[n]);
		if(Demo.M[n].level==3)
			AddMenu3(&Demo.M[n]);
		}

	Demo.M1[Demo.M1nb-1].NextMenu =NULL;
	Demo.MI2[Demo.M2nb-1].NextItem=NULL;
	Demo.MI3[Demo.M3nb-1].NextItem=NULL;

	window=U3D_Get(Scene,U3D_AMIGAWINDOW,0);		/* hack Scene's window */
	SetMenuStrip(window,Demo.M1);				/* add own Menus to Scene's window */
}
/*=================================================================*/
int main(int argc, char *argv[])
{
APTR Scene;
APTR Object;


	LIBOPEN(IntuitionBase,intuition.library,0)
	LIBOPEN(GfxBase,graphics.library,0)
	LIBOPEN(Microbe3DBase,Microbe3D.library,28)
#ifdef OS4
	LIBOPEN4(IIntuition,IntuitionBase)
	LIBOPEN4(IGraphics,GfxBase)
	LIBOPEN4(IMicrobe3D,Microbe3DBase)
#endif

	memset(&Demo,0,sizeof(struct DemoData));
	Demo.RotX		= 0.0;
	Demo.RotY		= 0.0;
	Demo.RotZ		= 0.0;
	Demo.SmoothAngle	= 80.0;
	Demo.WeldV		= 0.0;
	Demo.GridV		= 0.0;
	Demo.AngleN		= 0.0;
	Demo.EditX		= 1.0;
	Demo.EditY		= 1.0;
	Demo.EditZ		= 1.0;
	Demo.EditUV		= 0.0;
	Demo.EditThing	='I';
	Demo.width 		= 640;
	Demo.height		= 480;
	Demo.Infos		= TRUE;
	Demo.ShowStructure= FALSE;
	Demo.EditLnum	= 1;
	Demo.EditColorNum	= 1;

#ifdef OS3
	#define DEMOTITLE "Microbe3D View-OS3/68k"
#endif

#ifdef OS4
	#define DEMOTITLE "Microbe3D View-OS4/ppc"
#endif

#ifdef AROS
	#define DEMOTITLE "Microbe3D View-AROS/x86"
#endif

	Scene =U3D_EasyOpenScene(DEMOTITLE,Demo.width,Demo.height);

	Object=NiceRead(Scene,NULL);
	if (Object==NULL)	goto panic;

	DemoSetMenus(Scene);

	printf("Use Menus or keys. Press space for Editing Object/Instance/Light etc... \n"); 
	while(!Demo.Closed)
	{
		DemoProg(Scene);
		U3D_DrawScene(Scene);
		DemoDisplayInfos(Scene);
	}

panic:
	U3D_Delete(Scene);									/* close all */
#ifdef OS4
	LIBCLOSE4(IIntuition)
	LIBCLOSE4(IGraphics)
	LIBCLOSE4(IMicrobe3D)
#endif
	LIBCLOSE(IntuitionBase)
	LIBCLOSE(GfxBase)
	LIBCLOSE(Microbe3DBase)
	return 0;

}
/*==================================================================================*/






